import { Observable } from "rxjs";
import { AjaxResponse } from "rxjs/ajax";
import { ServerConfig, IGetParams, IContent, FileType, IContentProvider } from "@nteract/types";
import { JupyterAjaxResponse } from "./base";
/*********************************************
 * Contents API request and response payloads
 *********************************************/
/**
 * Creates an AjaxObservable for removing content.
 *
 * @param serverConfig The server configuration
 * @param path The path to the content
 *
 * @returns An Observable with the request response
 */
export declare const remove: (serverConfig: ServerConfig, path: string) => Observable<AjaxResponse>;
/**
 * Creates an AjaxObservable for getting content at a path
 *
 * @param serverConfig The server configuration
 * @param path The content to fetch
 * @param params type, format, content
 * @param params.type file type, one of 'file', 'directory', 'notebook'
 * @param params.format How file content should be returned, e.g. 'text', 'base64'
 * @param params.content Return content or not (0 => no content, 1 => content please)
 *
 * @returns An Observable with the request response
 */
export declare function get(serverConfig: ServerConfig, path: string, params?: Partial<IGetParams>): Observable<JupyterAjaxResponse<IContent<"file" | "directory" | "notebook">, string>>;
/**
 * Creates an AjaxObservable for renaming a file.
 *
 * @param serverConfig The server configuration
 * @param path The content to rename.
 * @param model The data to send in the server request
 *
 * @returns An Observable with the request response
 */
export declare function update<FT extends FileType>(serverConfig: ServerConfig, path: string, model: Partial<IContent<FT>>): Observable<AjaxResponse>;
/**
 * Creates an AjaxObservable for creating content
 *
 * @param serverConfig The server configuration
 * @param path The path to the content
 * @param model The data to send in the server request
 *
 * @returns An Observable with the request response
 */
export declare function create<FT extends FileType>(serverConfig: ServerConfig, path: string, model: Partial<IContent<FT>> & {
    type: FT;
}): Observable<AjaxResponse>;
/**
 * Creates an AjaxObservable for saving the file in the location specified by
 * name and path in the model.
 *
 * @param serverConfig  The server configuration
 * @param path The path to the content
 * @param model The data to send in the server request
 *
 * @returns An Observable with the request response
 */
export declare function save<FT extends FileType>(serverConfig: ServerConfig, path: string, model: Partial<IContent<FT>>): Observable<JupyterAjaxResponse<{
    [property: string]: string;
    path: string;
}, string>>;
/**
 * Creates an AjaxObservable for listing checkpoints for a given file.
 *
 * @param serverConfig The server configuration
 * @param path The content containing checkpoints to be listed.
 *
 * @returns An Observable with the request response
 */
export declare const listCheckpoints: (serverConfig: ServerConfig, path: string) => Observable<AjaxResponse>;
/**
 * Creates an AjaxObservable for creating a new checkpoint with the current state of a file.
 * With the default Jupyter FileContentsManager, only one checkpoint is supported,
 * so creating new checkpoints clobbers existing ones.
 *
 * @param serverConfig The server configuration
 * @param path The content containing the checkpoint to be created
 *
 * @returns An Observable with the request response
 */
export declare const createCheckpoint: (serverConfig: ServerConfig, path: string) => Observable<AjaxResponse>;
/**
 * Creates an AjaxObservable for deleting a checkpoint for a given file.
 *
 * @param  serverConfig The server configuration
 * @param  path The content containing the checkpoint to be deleted
 * @param  checkpointID ID of checkpoint to be deleted
 *
 * @returns An Observable with the request response
 */
export declare const deleteCheckpoint: (serverConfig: ServerConfig, path: string, checkpointID: string) => Observable<AjaxResponse>;
/**
 * Creates an AjaxObservable for restoring a file to a specified checkpoint.
 *
 * @param serverConfig The server configuration
 * @param path The content to restore to a previous checkpoint
 * @param checkpointID ID of checkpoint to be used for restoration
 *
 * @returns An Observable with the request response
 */
export declare const restoreFromCheckpoint: (serverConfig: ServerConfig, path: string, checkpointID: string) => Observable<AjaxResponse>;
/**
 * IContentProvider implementation for Jupyter content server.
 */
export declare const JupyterContentProvider: IContentProvider;
