"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const querystring_1 = __importDefault(require("querystring"));
const ajax_1 = require("rxjs/ajax");
const url_join_1 = __importDefault(require("url-join"));
const base_1 = require("./base");
const formURI = (path) => url_join_1.default("/api/contents/", path);
const formCheckpointURI = (path, checkpointID) => url_join_1.default("/api/contents/", path, "checkpoints", checkpointID);
/*********************************************
 * Contents API request and response payloads
 *********************************************/
/**
 * Creates an AjaxObservable for removing content.
 *
 * @param serverConfig The server configuration
 * @param path The path to the content
 *
 * @returns An Observable with the request response
 */
exports.remove = (serverConfig, path) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formURI(path), {
    method: "DELETE"
}));
/**
 * Creates an AjaxObservable for getting content at a path
 *
 * @param serverConfig The server configuration
 * @param path The content to fetch
 * @param params type, format, content
 * @param params.type file type, one of 'file', 'directory', 'notebook'
 * @param params.format How file content should be returned, e.g. 'text', 'base64'
 * @param params.content Return content or not (0 => no content, 1 => content please)
 *
 * @returns An Observable with the request response
 */
function get(serverConfig, path, params = {}) {
    let uri = formURI(path);
    const query = querystring_1.default.stringify(params);
    if (query.length > 0) {
        uri = `${uri}?${query}`;
    }
    // NOTE: If the user requests with params.content === 0
    // Then the response is IEmptyContent
    return ajax_1.ajax(base_1.createAJAXSettings(serverConfig, uri, { cache: false }));
}
exports.get = get;
/**
 * Creates an AjaxObservable for renaming a file.
 *
 * @param serverConfig The server configuration
 * @param path The content to rename.
 * @param model The data to send in the server request
 *
 * @returns An Observable with the request response
 */
function update(serverConfig, path, model) {
    return ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formURI(path), {
        body: model,
        headers: {
            "Content-Type": "application/json"
        },
        method: "PATCH"
    }));
}
exports.update = update;
/**
 * Creates an AjaxObservable for creating content
 *
 * @param serverConfig The server configuration
 * @param path The path to the content
 * @param model The data to send in the server request
 *
 * @returns An Observable with the request response
 */
function create(serverConfig, path, model) {
    return ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formURI(path), {
        body: model,
        headers: {
            "Content-Type": "application/json"
        },
        method: "POST"
    }));
}
exports.create = create;
/**
 * Creates an AjaxObservable for saving the file in the location specified by
 * name and path in the model.
 *
 * @param serverConfig  The server configuration
 * @param path The path to the content
 * @param model The data to send in the server request
 *
 * @returns An Observable with the request response
 */
function save(serverConfig, path, model) {
    return ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formURI(path), {
        body: model,
        headers: {
            "Content-Type": "application/json"
        },
        method: "PUT"
    }));
}
exports.save = save;
/**
 * Creates an AjaxObservable for listing checkpoints for a given file.
 *
 * @param serverConfig The server configuration
 * @param path The content containing checkpoints to be listed.
 *
 * @returns An Observable with the request response
 */
exports.listCheckpoints = (serverConfig, path) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formCheckpointURI(path, ""), {
    cache: false,
    method: "GET"
}));
/**
 * Creates an AjaxObservable for creating a new checkpoint with the current state of a file.
 * With the default Jupyter FileContentsManager, only one checkpoint is supported,
 * so creating new checkpoints clobbers existing ones.
 *
 * @param serverConfig The server configuration
 * @param path The content containing the checkpoint to be created
 *
 * @returns An Observable with the request response
 */
exports.createCheckpoint = (serverConfig, path) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formCheckpointURI(path, ""), {
    method: "POST"
}));
/**
 * Creates an AjaxObservable for deleting a checkpoint for a given file.
 *
 * @param  serverConfig The server configuration
 * @param  path The content containing the checkpoint to be deleted
 * @param  checkpointID ID of checkpoint to be deleted
 *
 * @returns An Observable with the request response
 */
exports.deleteCheckpoint = (serverConfig, path, checkpointID) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formCheckpointURI(path, checkpointID), {
    method: "DELETE"
}));
/**
 * Creates an AjaxObservable for restoring a file to a specified checkpoint.
 *
 * @param serverConfig The server configuration
 * @param path The content to restore to a previous checkpoint
 * @param checkpointID ID of checkpoint to be used for restoration
 *
 * @returns An Observable with the request response
 */
exports.restoreFromCheckpoint = (serverConfig, path, checkpointID) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, formCheckpointURI(path, checkpointID), {
    method: "POST"
}));
/**
 * IContentProvider implementation for Jupyter content server.
 */
exports.JupyterContentProvider = {
    remove(serverConfig, path) {
        return exports.remove(serverConfig, path);
    },
    get(serverConfig, path, params) {
        return get(serverConfig, path, params);
    },
    update(serverConfig, path, model) {
        return update(serverConfig, path, model);
    },
    create(serverConfig, path, model) {
        return create(serverConfig, path, model);
    },
    save(serverConfig, path, model) {
        return save(serverConfig, path, model);
    },
    listCheckpoints(serverConfig, path) {
        return exports.listCheckpoints(serverConfig, path);
    },
    createCheckpoint(serverConfig, path) {
        return exports.createCheckpoint(serverConfig, path);
    },
    deleteCheckpoint(serverConfig, path, checkpointID) {
        return exports.deleteCheckpoint(serverConfig, path, checkpointID);
    },
    restoreFromCheckpoint(serverConfig, path, checkpointID) {
        return exports.restoreFromCheckpoint(serverConfig, path, checkpointID);
    }
};
