"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const ajax_1 = require("rxjs/ajax");
const webSocket_1 = require("rxjs/webSocket");
const url_join_1 = __importDefault(require("url-join"));
const url_search_params_1 = __importDefault(require("@ungap/url-search-params"));
const base_1 = require("./base");
/**
 * Creates an AjaxObservable for listing running kernels.
 *
 * @param serverConfig The server configuration
 *
 * @returns An Observable with the request response
 */
exports.list = (serverConfig) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, "/api/kernels", { cache: false }));
/**
 * Creates an AjaxObservable for getting info about a kernel.
 *
 * @param serverConfig The server configuration
 * @param id The id of the kernel to fetch
 *
 * @returns An Observable with the request response
 */
exports.get = (serverConfig, id) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, `/api/kernels/${id}`, {
    cache: false
}));
/**
 * Creates an AjaxObservable for starting a kernel.
 *
 * @param serverConfig The server configuration
 * @param name The name of the kernel to start
 * @param path The path to start the kernel in
 *
 * @returns An Observable with the request response
 */
exports.start = (serverConfig, name, path) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, "/api/kernels", {
    headers: {
        "Content-Type": "application/json"
    },
    method: "POST",
    body: { path, name }
}));
/**
 * Creates an AjaxObservable for killing a kernel.
 *
 * @param serverConfig The server configuration
 * @param id The id of the kernel to kill
 *
 * @returns An Observable with the request response
 */
exports.kill = (serverConfig, id) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, `/api/kernels/${id}`, { method: "DELETE" }));
/**
 * Creates an AjaxObservable for interrupting a kernel.
 *
 * @param serverConfig The server configuration
 * @param id The id of the kernel to interrupt
 *
 * @returns An Observable with the request response
 */
exports.interrupt = (serverConfig, id) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, `/api/kernels/${id}/interrupt`, {
    method: "POST"
}));
/**
 * Creates an AjaxObservable for restarting a kernel.
 *
 * @param serverConfig The server configuration
 * @param id The id of the kernel to restart
 *
 * @returns An Observable with the request response
 */
exports.restart = (serverConfig, id) => ajax_1.ajax(base_1.createAJAXSettings(serverConfig, `/api/kernels/${id}/restart`, {
    method: "POST"
}));
/**
 * Creates a Websocket URL that can be used to initialize a
 * connection with a kernel.
 *
 * @param serverConfig The server configuration
 * @param kernelID The ID of the kernel to connect to
 * @param sessionID The ID of the session to connect as
 *
 * @returns A string with the fully formed Websocket URL
 */
exports.formWebSocketURL = (serverConfig, kernelID, sessionID) => {
    const params = new url_search_params_1.default();
    if (serverConfig.token) {
        params.append("token", serverConfig.token);
    }
    if (sessionID) {
        params.append("session_id", sessionID);
    }
    const q = params.toString();
    const suffix = q !== "" ? `?${q}` : "";
    const url = url_join_1.default(serverConfig.endpoint || "", `/api/kernels/${kernelID}/channels${suffix}`);
    return url.replace(/^http(s)?/, "ws$1");
};
/**
 * Creates a connection to a kernel with the given kernelID scoped under
 * a particular sessionID.
 *
 * @param serverConfig The server configuration
 * @param kernelID The ID of the kernel to connect to
 * @param sessionID The ID of the session to connect as
 *
 * @returns A websocket Subject that can be subscribed to
 */
exports.connect = (serverConfig, kernelID, sessionID) => {
    const wsSubject = webSocket_1.webSocket({
        url: exports.formWebSocketURL(serverConfig, kernelID, sessionID),
        protocol: serverConfig.wsProtocol,
        closeObserver: serverConfig.closeObserver
    });
    // Create a subject that does some of the handling inline for the session
    // and ensuring it's serialized
    return rxjs_1.Subject.create(rxjs_1.Subscriber.create((message) => {
        if (typeof message === "object") {
            const sessionizedMessage = Object.assign(Object.assign({}, message), { header: Object.assign({ session: sessionID }, message.header) });
            wsSubject.next(sessionizedMessage);
        }
        else {
            console.error("Message must be an object, the app sent", message);
        }
    }, (e) => wsSubject.error(e), () => wsSubject.complete()), // Subscriber
    // Subject.create takes a subscriber and an observable. We're only
    // overriding the subscriber here so we pass the subject on as an
    // observable as the second argument to Subject.create (since it's
    // _also_ an observable)
    wsSubject);
};
